@@ -64,6 +64,7 @@ class SmartPath(object):
 
             self.file_count = 0
             self.file_register = []
+            self.has_register = False
 
             if make_dir:
                 self.make_dir()
@@ -205,7 +206,7 @@ class SmartPath(object):
             print('Level \'{}\' is not in hierarchy!')
 
 
-    def cutoff_level(self, level):
+    def trim2level(self, level, remove='deeper_including'):
         '''
         Removes all levels that are higher or equal to given level.
 
@@ -214,15 +215,26 @@ class SmartPath(object):
         level : str
             String of the level that should be removed, together will all
             higher levels.
+        remove : str
+            what should be removed?
+            e.g. "deeper_including" removes the level itself, and deeper levels.
         '''
+
+        dict = {'deeper_including': '>=',
+                'deeper_excluding': '>',
+                'higher_including': '<=',
+                'higher_excluding': '<'}
+
         if level in self.hierarchy:
 
-            for h in copy.copy(self.hierarchy):
-                if h != level:
-                    self.remove_level(h)
-                else:
-                    self.remove_level(h)
-                    break
+            hierarchy = copy.copy(self.hierarchy)
+            level_ind = hierarchy.index(level)
+            indxs = np.array(range(len(hierarchy)))
+            cmd = 'np.array(hierarchy)[indxs {} {}].tolist()'.format(dict[remove], level_ind)
+            subset = eval(cmd)
+
+            for h in subset:
+                self.remove_level(h)
 
         else:
             print('Level \'{}\' is not in hierarchy!')
@@ -266,7 +278,7 @@ class SmartPath(object):
         return expand_full_path(self[level], files)
 
 
-    def search_files(self, level, pattern='', full_paths=False):
+    def search_files(self, level, pattern=('.'), full_paths=False):
         """
         Searches files meeting the regex pattern at level in the SmartPath
 
@@ -292,7 +304,7 @@ class SmartPath(object):
                                  full_paths=full_paths)[0]
 
 
-    def search_files_ts(self, level, pattern='',
+    def search_files_ts(self, level, pattern=('.'),
                         date_position=1, date_format='%Y%m%d_%H%M%S',
                         starttime=None, endtime=None, full_paths=False):
         """
@@ -344,39 +356,82 @@ class SmartPath(object):
         return df
 
 
-    def build_file_register(self, level=None, pattern='.'):
+    def build_file_register(self, down_to_level=None, up_to_level=None, pattern=('.')):
         """
         Builds a file register collecting files at all levels in the SmartPath.
 
         Parameters
         ----------
-        level : str, optional
+        down_to_level : str, optional
             deepest level that should be included in the file register
+        up_to_level : str, optional
+            highest level that should be included in the file register
         pattern : str tuple, optional
             strings defining search pattern for file search
             e.g. ('C1003', 'E048N012T6')
+
         """
         file_register = []
         file_count = 0
-        if level is None:
-            idx = self.hierarchy
+
+        # limit the hierarchy
+        if up_to_level is not None:
+            idx = self.hierarchy[self.hierarchy.index(up_to_level):]
+        elif down_to_level is not None:
+            idx = self.hierarchy[:self.hierarchy.index(down_to_level) + 1]
         else:
-            idx = self.hierarchy[0:self.hierarchy.index(level) + 1]
+            idx = self.hierarchy
+
+        # search files at each level
         for h in idx:
             if self.levels[h] is not None:
-                r, c = regex_file_search(
-                    self.build_levels(level=h),
-                    pattern, full_paths=True)
+                r, c = regex_file_search(self.build_levels(level=h), pattern, full_paths=True)
                 file_register += r
                 file_count += c
 
         self.file_register = file_register
         self.file_count = file_count
+        self.has_register = True
+
+
+    def get_disk_usage(self, unit=None, up_to_level=None, down_to_level=None, file_pattern=('.')):
+        '''
+        Computes the disk usage for each SmartPath and creates a Pandas DataFrame.
+
+        Parameters
+        ----------
+        unit : str
+            output unit of disk usage in bytes (e.g., "GB", "TB", ...)
+        down_to_level : str, optional
+            deepest level that should be included in the file register
+        up_to_level : str, optional
+            highest level that should be included in the file register
+        file_pattern : str tuple, optional
+            strings defining file pattern that are included in disk usage sums
+            e.g. ('C1003', 'E048N012T6')
+
+        Returns
+        -------
+        Number
+            disk usage of all files along the SmartPath
+
+        '''
+
+
+        self.build_file_register(down_to_level=down_to_level, up_to_level=up_to_level,
+                                 pattern=file_pattern)
+
+        # compute size of directory
+        nbytes = sum([os.path.getsize(filepath) for filepath in self.file_register])
+        dir_size = transform_bytes(nbytes, unit=unit)
+
+        return np.round(dir_size, 3)
 
 
 class NullSmartPath(SmartPath):
     '''
     Class for non-exiting paths. Helps to avoid errors.
+
     '''
     def __init__(self):
         super(NullSmartPath, self).__init__({}, [], make_dir=False)
@@ -399,8 +454,8 @@ class SmartTree(object):
             List defining the order of the levels
         make_dir : bool, optional
             creates the root directory
-        '''
 
+        '''
         if not os.path.exists(root) and make_dir == False:
             raise OSError('Given directory for attribute \'root\', '
                           '\'{}\', does not exist!'.format(root))
@@ -411,6 +466,7 @@ class SmartTree(object):
         self.dir_count = 0
         self.file_count = 0
         self.file_register = []
+        self.has_register = False
 
         if make_dir:
             if not os.path.exists(self.root):
@@ -427,6 +483,15 @@ class SmartTree(object):
         return self.get_smartpath(pattern)
 
 
+    def print_root(self):
+        '''
+        Function to print nicely the root to screen.
+
+        '''
+
+        print(self.root)
+
+
     def print_all_dirs(self):
         '''
         Nice function to print nicely all directories to screen.
@@ -446,11 +511,11 @@ class SmartTree(object):
 
     def print_collect_level(self, level, pattern=None, unique=False):
         '''
-        Nice function to print nicely output from collect_level()
+        Nice function to print nicely output from collect_level_string()
         '''
 
-        print('\n'.join(self.collect_level(level, pattern=pattern,
-                                           unique=unique)))
+        print('\n'.join(self.collect_level_string(level, pattern=pattern,
+                                                  unique=unique)))
 
 
     def print_collect_level_topnames(self, level, pattern=None, unique=True):
@@ -520,6 +585,78 @@ class SmartTree(object):
         return sorted(list(self.dirs.keys()))
 
 
+    def get_disk_usage(self, unit='KB',
+                       group_by=[],
+                       file_pattern=('.'),
+                       total=False):
+        '''
+        Computes the disk usage for each SmartPath and creates a Pandas DataFrame.
+
+        Parameters
+        ----------
+        unit : str, optional
+            output unit of disk usage in bytes (e.g., "GB", "TB", ...)
+        group_by : list, optional
+            list of levels forming groups, delivering disk usage sums
+            e.g. ['tile', 'var']
+        file_pattern : str tuple, optional
+            strings defining file pattern that are included in disk usage sums
+            e.g. ('M2019', 'SSM------')
+        total : bool, optional
+            returns the total disk usage for the root
+
+        Returns
+        -------
+        DataFrame
+            Pandas DataFrame containing the disk usage per SmartPath and the directory
+            hierarchy as columns (without the root directory path)
+        '''
+
+        # hierarchy settings
+        rootless_hierarchy = self.hierarchy[1:]
+        n = len(rootless_hierarchy)
+
+        # build SmartTree() disk usage table
+        dir_size_table = []
+
+        # loop over all levels of the hierarchy
+        for i, level in enumerate(rootless_hierarchy):
+            smpts_at_level = self.collect_level_smartpath(level, unique=True)
+
+            # loop over trimmed paths
+            for smpt in smpts_at_level:
+                dir_elems = smpt.get_dir().replace(self.root, '').split(os.sep)
+                dir_elems = map(lambda x: x.replace(os.sep, ''), dir_elems)
+                dir_elems = [dir_elem for dir_elem in dir_elems if dir_elem != '']
+
+                # fill up levels below with None
+                remaining_levels = n - (i + 2) + 1
+                dir_elems += [None] * remaining_levels
+
+                filepaths = smpt.search_files(level, pattern=file_pattern, full_paths=True)
+                filepaths = [filepath for filepath in filepaths if filepath in self.file_register]
+
+                # get the disk usage of all files at the current level
+                nbytes = sum([os.path.getsize(filepath) for filepath in filepaths])
+                disk_usage = transform_bytes(nbytes, unit=unit)
+
+                dir_elems.append(disk_usage)
+                dir_size_table.append(dir_elems)
+
+        # create Pandas DataFrame
+        df = pd.DataFrame(data=dir_size_table, columns=self.hierarchy[1:] + ['du'])
+
+        # return total disk usage
+        if total:
+            return pd.DataFrame(data=[[self.collect_level_topnames('root')[0], df['du'].sum()]], columns=['root', 'du'])
+        # return disk usage of each SmartPaths
+        elif group_by == []:
+            return df
+        # return disk usage summed over grouped SmartPaths
+        else:
+            return df.groupby(group_by).sum()
+
+
     def count_dirs(self):
         '''
         Sets the dir_count the SmartTree
@@ -564,9 +701,10 @@ class SmartTree(object):
             return self.dirs[matching_paths[0]]
 
 
-    def collect_level(self, level, pattern=None, unique=False):
+    def collect_level_string(self, level, pattern=None, unique=False):
         '''
-        Returns a list of paths at given level.
+        Returns a list of paths at given level,
+        and matching a given pattern.
 
         Parameters
         ----------
@@ -583,23 +721,57 @@ class SmartTree(object):
             list of paths at given level, matching the given pattern
         '''
 
+        result = self.collect_level_smartpath(level=level, pattern=pattern, unique=unique)
+
+        strings = [x.get_dir() for x in result]
+
+        return strings
+
+
+    def collect_level_smartpath(self, level, pattern=None, unique=False):
+        '''
+        Returns a list of Smartpaths reaching a given level,
+        and matching a given pattern.
+
+        Parameters
+        ----------
+        level : str
+            name of level in hierarchy
+        pattern : str tuple, optional
+            strings defining search pattern for path search
+            e.g. ('C1003', 'E048N012T6')
+        unique : bool, optional
+            if set, a list of unique paths is returned
+        Returns
+        -------
+        list of SmartPaths
+            list of paths at given level, matching the given pattern
+        '''
+
         result = []
 
-        for _, smartpath in self.dirs.items():
+        for _, elem in self.dirs.items():
+            smartpath = copy.deepcopy(elem)
             if smartpath.levels[level] is not None:
-
                 if pattern is not None:
                     rpattern = patterns_2_regex(pattern)
                     regex = re.compile(rpattern)
                     if regex.match(smartpath.levels[level]):
-                        result.append(smartpath[level])
+                        smartpath.trim2level(level, remove='deeper_excluding')
+                        result.append(smartpath)
                 else:
-                    result.append(smartpath[level])
+                    smartpath.trim2level(level, remove='deeper_excluding')
+                    result.append(smartpath)
 
+        result = np.array(result)
         if unique:
-            result = list(set(result))
+            # remove duplicates after trimming the tree
+            _, uni_idx = np.unique([x.get_dir() for x in result], return_index=True)
+            result = result[uni_idx]
 
-        return sorted(result)
+        sort_ind = np.argsort([x.get_dir() for x in result])
+
+        return result[sort_ind].tolist()
 
 
     def collect_level_topnames(self, level, pattern=None, unique=True):
@@ -622,7 +794,7 @@ class SmartTree(object):
             list of folder-topnames at given level, matching the given pattern
         '''
 
-        paths = self.collect_level(level, pattern=pattern, unique=unique)
+        paths = self.collect_level_string(level, pattern=pattern, unique=unique)
 
         topnames = [x.split(os.sep)[-1] for x in paths]
 
@@ -696,7 +868,7 @@ class SmartTree(object):
 
         branch = copy.deepcopy(self)
 
-        branch_path = self.collect_level(level, pattern=pattern, unique=True)
+        branch_path = self.collect_level_string(level, pattern=pattern, unique=True)
 
         if len(branch_path) == 0:
             warnings.warn('trim2branch(): No matches for "pattern" at "level"!')
@@ -709,7 +881,7 @@ class SmartTree(object):
                 if not branch_path[0] in d:
                     branch.remove_smartpath(d)
                 else:
-                    branch.dirs.get(d).cutoff_level(level)
+                    branch.dirs.get(d).trim2level(level, remove="higher_including")
                     branch.dirs.get(d).base_onto_root(branch_path[0])
 
             # update dir_count
@@ -736,6 +908,7 @@ class SmartTree(object):
 
             branch.file_register = file_register
             branch.file_count = file_count
+            branch.has_register = True
 
             return branch
 
@@ -782,7 +955,7 @@ class SmartTree(object):
             base_folder = self.collect_level_topnames('root')[0]
         else:
             branch = self.trim2branch(level, level_pattern, register_file_pattern=file_pattern)
-            source_dir = branch.collect_level('root', level_pattern, unique=True)[0]
+            source_dir = branch.collect_level_string('root', level_pattern, unique=True)[0]
             base_folder = branch.collect_level_topnames('root')[0]
 
         target_dir = os.path.join(target_dir, base_folder)
@@ -940,7 +1113,7 @@ def build_smarttree(root,
         # to register only files down to target level
         if trim_level is None:
             if register_file_pattern is not None and target_level is not None:
-                smart_path.build_file_register(level=target_level,
+                smart_path.build_file_register(down_to_level=target_level,
                                                pattern=register_file_pattern)
                 file_register += smart_path.file_register
 
@@ -960,6 +1133,9 @@ def build_smarttree(root,
         smart_tree = smart_tree.trim2branch(trim_level, pattern=trim_pattern,
                                             register_file_pattern=register_file_pattern)
 
+    if register_file_pattern is not None:
+        smart_tree.has_register = True
+
     return smart_tree
 
 
@@ -1155,6 +1331,16 @@ def copy_tree(source, dest, file_pattern=None, overwrite=False):
                     shutil.copy2(os.path.join(root, file), file2write)
 
 
+def transform_bytes(bytes, unit='KB'):
+    scale_factor_dict = {"TB": 1e-12, "GB": 1e-9, "MB": 1e-6, "KB": 1e-3}
+    scale_factor = 1.
+    if unit is not None:
+        try:
+            scale_factor = scale_factor_dict[unit.upper()]
+        except KeyError:
+            raise KeyError('Unit {} unknown.'.format(unit))
+
+    return bytes*scale_factor
 
 if __name__ == '__main__':
     pass
\ No newline at end of file
